@@ -13,17 +13,16 @@
 
 /* local prototypes */
 int save_part_as_pir(int debut, int fin, char **sequence, char **seqname, 
-	int *sel_seqs, int tot_seqs, char *fname, int *withU, int *empty_seq,
-	int *num_longest, int protein, int align_everything);
+	int *sel_seqs, int tot_seqs, char *fname, int *withU, int *num_longest, int protein);
 int replace_align_part(SEA_VIEW *view, int debut, int fin, char *fname, int withU, int num_longest,
-	int has_phylip_format, int align_everything);
+	int has_phylip_format, int *tmpselseqs);
 void align_selected_parts(SEA_VIEW *view, int align_algorithm, int align_everything);
 int calc_gap_sites(char *old_seq, char *new_seq, int lold, int lnew, 
 	gap_site *gap_sites, int maxsites);
 int insert_gaps_new_align(char **seq, int site, int number, int numseqs, 
 	int lseqs);
-int reset_stars(SEA_VIEW *view, int debut, int lpart, char **seq, int lfrag, int align_everything);
-int confirm_refer_seq(int num_longest, SEA_VIEW *view);
+int reset_stars(SEA_VIEW *view, int debut, int lpart, char **seq, int lfrag, int *tmpselseqs);
+int confirm_refer_seq(int num_longest, SEA_VIEW *view, int *tmpselseqs);
 int calc_msa_command(SEA_VIEW *view, char *base_fname, char *command);
 void cre_align_opt_menu(SEA_VIEW *view, Fl_Menu_Button *obj, props_menu_parts *parts);
 void align_opt_callback(Fl_Widget *ob, void *data);
@@ -40,25 +39,17 @@ void init_msa_algos(void);
 extern int insert_gaps_at(SEA_VIEW *view, int seq, int site, int total);
 extern int insert_gap_all_comments(int numgaps, int pos,  SEA_VIEW *view);
 extern void del_gap_only_sites(SEA_VIEW *view);
-extern char *get_res_value(char *name, char *def_value);
-extern int int_res_value(char *name, int def_value);
+extern char *get_res_value(const char *name, const char *def_value);
+extern int int_res_value(const char *name, int def_value);
 extern int set_res_value(const char *name, const char *value);
-extern void delete_res_value(char *name);
+extern void delete_res_value(const char *name);
 extern void save_resources(void);
 #ifdef WIN32
-extern char *MG_win32_file_chooser(const char* message,
-        const char* pattern,
-        const char* fname,
-        const char* defext,
-        bool save, int *pfilterindex);
 extern "C" { int mysystem(const char *command); }
 #elif defined(__APPLE__)
-extern char* MAC_file_chooser(const char* message, const char* pat, const char* fname);
 extern "C" {
 	int my_system_macho(char *command, char *base_fname);
 	}
-#else
-void change_attr(Fl_Window *w);
 #endif
 
 
@@ -67,13 +58,13 @@ extern gap_site gap_sites[];
 
 
 int save_part_as_pir(int debut, int fin, char **sequence, char **seqname, 
-	int *sel_seqs, int tot_seqs, char *fname, int *withU, int *empty_seq,
-	int *num_longest, int protein, int align_everything)
+	int *sel_seqs, int tot_seqs, char *fname, int *withU,
+	int *num_longest, int protein)
 /* returns TRUE iff error */
 {
 FILE *out;
-int num, pos, l_line, retval, maxlen, seqlen, current;
-char line[90];
+int num, pos, l_line, retval, maxlen, seqlen, current, empty_seq;
+char line[90], lname[25];
 
 out = fopen(fname,"w");
 if( out == NULL ) return TRUE;
@@ -81,16 +72,20 @@ retval = TRUE;
 *withU = FALSE;
 maxlen = 0; current = 0;
 for(num = 0; num < tot_seqs; num++) {
-	if(! (align_everything ||  sel_seqs[num] ) ) continue;
-	sprintf(line, "%d_%s", ++current, seqname[num]);
-	fprintf(out, ">%.10s\n", line);
-	if(ferror(out)) goto fin;
-	*empty_seq = TRUE;
+	if(! sel_seqs[num] ) continue;
+	empty_seq = TRUE;
+	sprintf(line, "%d_%.10s", ++current, seqname[num]);
+	sprintf(lname, ">%.10s\n", line);
 	l_line = 0;
 	seqlen = 0;
 	for ( pos = debut - 1 ; pos < fin; pos++) {
 		if( sequence[num][pos] == 0 ) break;
 		if( sequence[num][pos] == '-' ) continue;
+		if(empty_seq) {
+			fputs(lname, out);
+ 			if(ferror(out)) goto fin;
+			empty_seq = FALSE;
+			}
 		if(l_line >= 70) {
 			line[l_line] = 0;
 			if(!*withU) *withU = (strchr(line, 'U') != NULL);
@@ -101,7 +96,6 @@ for(num = 0; num < tot_seqs; num++) {
 			}
 		line[l_line++] = sequence[num][pos];
 		if(sequence[num][pos] == '*') line[l_line - 1] = 'X';
-		*empty_seq = FALSE;
 		}
 	line[l_line] = 0;
 	seqlen += l_line;
@@ -110,8 +104,8 @@ for(num = 0; num < tot_seqs; num++) {
 		}
 	if(!*withU) *withU = (strchr(line, 'U') != NULL);
 	fprintf(out, "%s\n", line);
-	if( *empty_seq ) strcpy(fname, seqname[num]);
-	if(ferror(out) || *empty_seq) goto fin;
+	if(ferror(out)) goto fin;
+	if(empty_seq) sel_seqs[num] = FALSE;
 	}
 retval = FALSE;
 fin:
@@ -120,11 +114,23 @@ return retval;
 }
 
 
+#if ! ( defined(WIN32)  ||  defined(__APPLE__) )
+void align_listener(int fd, void *data)
+{
+char line[100];
+int l = read(fd, line, sizeof(line));
+if(l == 0) *(int *)data = TRUE;
+}
+#endif
+
+
 void align_selected_parts(SEA_VIEW *view, int align_algorithm, int align_everything)
 {
-int debut, fin, status, withU, empty_seq, num_longest, l;
+int debut, fin, status, withU, *tmpselseqs, num_longest, l;
 char *p;
-static char base_fname[100], commande[300];
+char base_fname[100];
+static char commande[600];
+FILE *in;
 
 #if ! ( defined(WIN32)  ||  defined(__APPLE__) )
 /* check for presence of xterm + seaview_align.sh */
@@ -135,7 +141,7 @@ static char base_fname[100], commande[300];
 	if(status != 0) {
 		fl_message("Alignment operation is impossible because\n"
 		  "one of the programs xterm, seaview_align.sh\n"
-		  "is missing");
+		  "is missing from PATH or seaview's directory");
 		((Fl_Menu_ *)view->menu_edit)->mode(ALIGN_ALL_SEQS, FL_MENU_INACTIVE);
 		((Fl_Menu_ *)view->menu_edit)->mode(ALIGN_SELECTION, FL_MENU_INACTIVE);
 		return;
@@ -147,11 +153,6 @@ if( (!align_everything) && (view->active_region == NULL || view->active_region->
 		"and some selected sequences");
 	return;
 	}
-#if !( defined(WIN32) || defined(__APPLE__) )
-/* mettre fenetres en Backing Store State: WhenMapped */
-change_attr( view->DNA_obj->window() ); 
-change_attr( view->dnawin ); 
-#endif
 if(align_everything) {
 	debut = 1;
 	fin = view->seq_length;
@@ -171,54 +172,67 @@ convertir en minuscules car clustalw n'accepte pas les majuscules ds filenames
 */
 p = base_fname - 1; while( *(++p) != 0) *p = tolower( *p);
 sprintf(commande, "%s.pir", base_fname);
-/* allonger les seqs plus courtes que la region traitee */
+/* allonger les seqs plus courtes que la region traitee sans compter comme modif alignment*/
+int old_modif_but_not_saved = view->modif_but_not_saved;
 for(l = 0; l < view->tot_seqs; l++) {
 	if(!(align_everything || view->sel_seqs[l]) ) continue;
 	if(fin > view->each_length[l]) insert_gaps_at(view, l + 1, 
 			view->each_length[l] + 1, fin - view->each_length[l]);
 	}
+set_seaview_modified(view, old_modif_but_not_saved);
+/* prepare list of empty seqs */
+tmpselseqs = (int *)malloc(view->tot_seqs * sizeof(int));
+if(align_everything)
+	memset(tmpselseqs, 1, view->tot_seqs * sizeof(int));
+else
+	memcpy(tmpselseqs, view->sel_seqs, view->tot_seqs * sizeof(int));
 status = save_part_as_pir(debut, fin, view->sequence, view->seqname, 
-	view->sel_seqs, view->tot_seqs, commande, &withU, &empty_seq, 
-	&num_longest, view->protein, align_everything);
+	tmpselseqs, view->tot_seqs, commande, &withU, &num_longest, view->protein);
 if(view->protein) withU = FALSE;
-if(empty_seq) {
-	fl_alert("Cannot process sequence(s)\n%s\n"
-		 "containing only gaps", commande);
-	}
-if( (!empty_seq) && status) {
+if(status) {
 	fl_alert("Cannot write sequences to filename\n%s", commande);
+	free(tmpselseqs);
 	return;
 	}
-if(!(empty_seq || align_everything) ) num_longest = confirm_refer_seq(num_longest, view);
-if(num_longest == -1 || empty_seq) {
+if(!align_everything ) num_longest = confirm_refer_seq(num_longest, view, tmpselseqs);
+if(num_longest == -1 ) {
 	sprintf(commande, "%s.pir", base_fname);
 	remove(commande);
+	free(tmpselseqs);
 	return;
 	}
 my_watch_cursor(view->dnawin);
 
 status = calc_msa_command(view, base_fname, commande);
-if(status) return;
+if(status) {
+	free(tmpselseqs);
+	return;
+	}
 #ifdef WIN32
-status = mysystem(commande);
+	status = mysystem(commande);
 #elif defined(__APPLE__)
-status = my_system_macho(commande, base_fname);
+	status = my_system_macho(commande, base_fname);
 #else
-status = system(commande);
-if( status == 0 ) {
+	in = popen(commande, "r");
+	if(in != NULL) {
+		int align_ended = FALSE;
+		Fl::add_fd(fileno(in), align_listener, &align_ended);
+		while(!align_ended) Fl::wait();
+		Fl::remove_fd(fileno(in));
+		pclose(in);
+		}
 	sprintf(commande, "%s.status", base_fname);
-	FILE *in = fopen(commande, "r");
+	in = fopen(commande, "r");
 	if(in != NULL) {
 		fgets(commande, sizeof(commande), in);
 		if( strcmp(commande, "success\n") != 0) status = 1;
 		fclose(in);
 		}
 	else	status = 1;
-	}
 #endif
 sprintf(commande, "%s.out", base_fname);
 if( status == 0 ) {
-	FILE *in = fopen(commande, "r");
+	in = fopen(commande, "r");
 	if(in != NULL) {
 		fseek(in, 0, SEEK_END);
 		status = (ftell(in) < 10);
@@ -227,14 +241,15 @@ if( status == 0 ) {
 	else	status = 1;
 	}
 if( status != 0 )
-	fl_message("Alignment operation ignored");
+	fl_message("%s: alignment operation ignored", extract_filename(view->masename));
 else	{
 	status = replace_align_part(view, debut, fin, commande, withU, num_longest,
-		 FALSE, 
-		align_everything );
+		 FALSE, tmpselseqs );
 	if(status) fl_alert("Not enough memory\nor\n" 
 		"max sequence length reached");
+	else if(align_everything) del_gap_only_sites(view);
 	}
+free(tmpselseqs);
 sprintf(commande, "%s.pir", base_fname);
 remove(commande);
 sprintf(commande, "%s.out", base_fname);
@@ -284,7 +299,7 @@ return tot_sites;
 
 
 int replace_align_part(SEA_VIEW *view, int debut, int fin, char *fname, 
-	int withU, int num_longest, int has_phylip_format, int align_everything)
+	int withU, int num_longest, int has_phylip_format, int *tmpselseqs)
 /* returns TRUE if error, FALSE if ok */
 {
 int num, lfrag, lpart, i, col, rang, retval, newlength, lfrag2, total;
@@ -293,7 +308,8 @@ int (*calc_color_function)(int);
 int totgapsites, num1, site, l_copy, res;
 char *tmp;
 
-total = (align_everything ? view->tot_seqs : view->tot_sel_seqs);
+total = 0;
+for(num = 0; num < view->tot_seqs; num++) if(tmpselseqs[num]) total++;
 view->cursor_in_comment = FALSE;
 view->cursor_seq = view->first_seq;
 /* lecture de l'alignement multiple produit par clustalw */
@@ -320,11 +336,11 @@ if(withU) {
 	}	
 lfrag = strlen(seq[0]); /* long alignement de clustalw */
 lpart = fin - debut + 1; /*long region traitee de l'ancien alignement multiple*/
-if( reset_stars(view, debut, lpart, seq, lfrag, align_everything) ) return TRUE;
+if( reset_stars(view, debut, lpart, seq, lfrag, tmpselseqs) ) return TRUE;
 /* num1 = rang dans align clustalw de num_longest dans align multiple */
 num1 = -1;
 for(i = 0; i < view->tot_seqs; i++) {
-	if( !(align_everything || view->sel_seqs[i]) ) continue;
+	if( ! tmpselseqs[i] ) continue;
 	num1++;
 	if( i == num_longest) break;
 	}
@@ -384,7 +400,7 @@ for(site = 0; site < totgapsites; site++) {
 /* copie des sequences de clustalw vers alignement multiple */
 rang = -1;
 for(num = 0; num < view->tot_seqs; num++) {
-	if(  ! (align_everything || view->sel_seqs[num]) ) continue;
+	if(  ! tmpselseqs[num] ) continue;
 	rang++;
 /* on met les nouvelles seqs */ 
 	l_copy = FL_min(lpart, lfrag);
@@ -411,8 +427,7 @@ for(num = 0; num < view->tot_seqs; num++) {
 	}
 /* mettre a jour horsli */
 update_current_seq_length(newlength, view);
-if(align_everything) del_gap_only_sites(view);
-view->modif_but_not_saved = TRUE;
+set_seaview_modified(view, TRUE);
 retval = FALSE;
 
 fin:
@@ -437,14 +452,14 @@ return lseqs + number;
 }
 
 
-int reset_stars(SEA_VIEW *view, int debut, int lpart, char **seq, int lfrag, int align_everything)
+int reset_stars(SEA_VIEW *view, int debut, int lpart, char **seq, int lfrag, int *tmpselseqs)
 {
 int oldseq, newseq;
 char *p, *q;
 
 newseq = -1;
 for(oldseq = 0; oldseq < view->tot_seqs; oldseq++) {
-	if(!(align_everything || view->sel_seqs[oldseq])) continue;
+	if(! tmpselseqs[oldseq] ) continue;
 	newseq++;
 	p = view->sequence[oldseq] + debut - 2;
 	q = seq[newseq] - 1;
@@ -462,7 +477,7 @@ return FALSE;
 }
 
 
-int confirm_refer_seq(int num_longest, SEA_VIEW *view)
+int confirm_refer_seq(int num_longest, SEA_VIEW *view, int *tmpselseqs)
 {
 static Fl_Window *form;
 static int first = TRUE;
@@ -493,7 +508,7 @@ if(first) {
 browser_noms->clear();
 lnum = 0;
 for(i = 0; i < view->tot_seqs; i++) {
-	if(!view->sel_seqs[i]) continue;
+	if(!tmpselseqs[i]) continue;
 	lnum++;
 	browser_noms->add(view->seqname[i]);
 	if(i == num_longest) browser_noms->value(lnum);
@@ -511,7 +526,7 @@ Fl::flush();
 
 lnum = browser_noms->value();
 for(i = 0; i < view->tot_seqs; i++) {
-		if(!view->sel_seqs[i]) continue;
+		if(!tmpselseqs[i]) continue;
 		if(strcmp(view->seqname[i], browser_noms->text(lnum)) == 0) {
 			num_longest = i;
 			break;
@@ -523,7 +538,7 @@ return num_longest;
 
 int calc_msa_command(SEA_VIEW *view, char *base_fname, char *command)
 {
-char algo_name[50];
+char algo_name[200];
 char local[300], tmp[300], *p, *optargs;
 int status;
 
@@ -541,8 +556,8 @@ status = (p == NULL);
 status = (get_full_path(algo_name) == NULL);
 #endif
 if(status) {
-	sprintf(tmp, "Alignment is impossible because program  '%s'  is not found", algo_name);
-	fl_message(tmp);
+	fl_message("Alignment is impossible because program  '%s'  is not found\n"
+	"in PATH or seaview's directory", algo_name);
 	return 1;
 	}
 	
@@ -582,8 +597,10 @@ if((p=strchr(local, '>')) != NULL) {
 	memmove(p+3, p+1, strlen(p+1) + 1);
 	memcpy(p, "'>'", 3);
 	}
-sprintf(command, "xterm -T \"%s alignment\" -n %s -sb -sl 2000 -e seaview_align.sh %s %s %s ", 
-	algo_name, algo_name, algo_name, base_fname, local);
+p = strdup(get_full_path("seaview_align.sh"));
+sprintf(command, "xterm -T \"%s alignment\" -n %s -sb -sl 2000 -e %s %s %s %s ", 
+	algo_name, algo_name, p, get_full_path(algo_name), base_fname, local);
+free(p);
 #endif
 return 0;
 }
@@ -742,21 +759,19 @@ else ((Fl_Menu_Item *)menu_props->menu() + num)->deactivate();
 save_resources();
 }
 
+#include "FL/Fl_Native_File_Chooser.H"
 
 void choose_align_callback(Fl_Widget *ob, void *data)
 {
-#ifdef __APPLE__
-char *name = MAC_file_chooser("Select external alignment program", NULL, "");
-#elif defined(WIN32)
-char *name = MG_win32_file_chooser("Select external alignment program", 
-	"program (.exe)\0*.exe\0any\0*.*\0", "", NULL, FALSE, NULL);
-#else
-char *name = fl_file_chooser("Select external alignment program", NULL, "", 0);
+Fl_Native_File_Chooser *chooser = new Fl_Native_File_Chooser();
+chooser->type(Fl_Native_File_Chooser::BROWSE_FILE);   // let user browse a single file
+chooser->title("Select external alignment program");                        
+#if defined(WIN32)
+	chooser->filter("*.exe");                 
 #endif
-if(name != NULL) {
-	((Fl_Input*)data)->value(name);
-	((Fl_Input*)data)->position(strlen(name));
-	}
+if ( chooser->show() != 0 ) return;
+((Fl_Input*)data)->value(chooser->filename());
+((Fl_Input*)data)->position(strlen(chooser->filename()));
 }
 
 
@@ -879,23 +894,4 @@ if(count < 2) {
 }
 
 
-#if ! ( defined(WIN32) || defined(__APPLE__) )
-
-void change_attr(Fl_Window *w)
-{
-XSetWindowAttributes attr;
-XWindowAttributes current;
-
-XGetWindowAttributes(fl_display, fl_xid(w), &current);
-if(current.backing_store != NotUseful) return;
-attr.backing_store = WhenMapped;
-XChangeWindowAttributes(fl_display, fl_xid(w), CWBackingStore, &attr);
-// il faut la redessiner pour que ca fasse effet
-w->redraw(); 
-Fl::flush();
-}
-
-#endif
-
-
 
