@@ -330,8 +330,6 @@ if(view->phylipwidnames < 10) {
 	}
 
 view->format_for_save = (known_format)saveformat->value();
-item = (Fl_Menu_Item *)menu_props->menu() + props_parts->saveformat;
-(item+view->format_for_save)->setonly();
 
 view->alignment_algorithm = alignment_algo->value();
 item = (Fl_Menu_Item *)menu_props->menu() + props_parts->clustalopt;
