@@ -33,7 +33,7 @@ void update_comments_callback(Fl_Widget *ob, void *data);
 int load_comments(SEA_VIEW *view, Fl_Input *input, Fl_Widget *name);
 char *cre_consensus(SEA_VIEW *view, char *newname);
 Fl_Widget *cre_adjusted_button(int x, int y, int *w, int h, 
-	char *label);
+	const char *label);
 
 
 /* external prototypes */
@@ -161,7 +161,7 @@ ob->window()->hide();
 
 
 Fl_Widget *cre_adjusted_button(int x, int y, int *w, int h, 
-	char *label)
+	const char *label)
 {
 Fl_Button *obj;
 const int lsize = FL_NORMAL_SIZE;
@@ -297,12 +297,15 @@ char **new_s, **new_n, **new_c;
 num = view->tot_seqs + 1;
 new_s = (char **)realloc(view->sequence, num * sizeof(char *));
 if(new_s != NULL) view->sequence = new_s;
-new_c = (char **)realloc(view->comments, num * sizeof(char *));
-if(new_c != NULL) view->comments = new_c;
+if(view->comments != NULL) {
+	new_c = (char **)realloc(view->comments, num * sizeof(char *));
+	if(new_c != NULL) view->comments = new_c;
+	else goto nomem;
+	}
 new_n = (char **)realloc(view->seqname, num * sizeof(char *));
 if(new_n != NULL) view->seqname = new_n;
 
-if(new_s == NULL || new_n == NULL || new_c == NULL) goto nomem;
+if(new_s == NULL || new_n == NULL) goto nomem;
 if(lenseq > view->max_seq_length) {
 	lenseq = view->max_seq_length;
 	newseq[lenseq] = 0;
@@ -311,8 +314,10 @@ if(lenseq > view->max_seq_length) {
 if( (view->seqname[num-1] = (char *)malloc(strlen(newname)+1)) == NULL) 
 	goto nomem;
 strcpy(view->seqname[num-1], newname);
-if( (view->comments[num-1] = (char *)malloc(3)) == NULL) goto nomem;
-strcpy(view->comments[num-1], ";\n");
+if(view->comments != NULL) {
+	if( (view->comments[num-1] = (char *)malloc(3)) == NULL) goto nomem;
+	strcpy(view->comments[num-1], ";\n");
+	}
 if( (view->sequence[num-1] = (char *)malloc(view->max_seq_length + 1)) == NULL) 
 	goto nomem;
 memcpy(view->sequence[num-1], newseq, lenseq+1);
@@ -323,7 +328,7 @@ if(view->masename == NULL) {
 else newname = view->masename;
 if(view->tot_seqs == 0) {
 	init_dna_scroller(view, 1, newname, is_a_protein_seq(newseq), NULL);
-	view->modif_but_not_saved = TRUE;
+	set_seaview_modified(view, TRUE);
 	view->DNA_obj->redraw();
 	Fl_Menu_Item *items = 
 		(Fl_Menu_Item *)((Fl_Menu_ *)view->menu_file)->menu();
@@ -375,7 +380,7 @@ view->tot_seqs = num;
 view->cursor_seq = num;
 view->cursor_site = 1;
 view->first_site = 1;
-view->modif_but_not_saved = TRUE;
+set_seaview_modified(view, TRUE);
 ((Fl_Slider*)view->horsli)->value(1);
 set_tot_lines(view, view->tot_lines);
 view->first_seq = FL_max(num - view->tot_lines + 1, 1);
@@ -435,7 +440,7 @@ if(first) {
 
 	comments_name = new Fl_Box(FL_DOWN_BOX, fin, 5, 
 		top_group->x() +top_group->w() - fin, 25, "");
-	comments_name->align(FL_ALIGN_CENTER);
+	comments_name->align(FL_ALIGN_LEFT | FL_ALIGN_CLIP | FL_ALIGN_INSIDE);
 	comments_name->labelfont(FL_COURIER);
 	comments_name->labelsize(FL_NORMAL_SIZE);
 
@@ -504,7 +509,7 @@ fin += width + 5;
 
 editdata->name = new Fl_Box(FL_DOWN_BOX, fin, 5, 
 	top_group->x() +top_group->w() - fin, 25, "");
-editdata->name->align(FL_ALIGN_CENTER);
+editdata->name->align(FL_ALIGN_LEFT | FL_ALIGN_CLIP | FL_ALIGN_INSIDE);
 editdata->name->labelfont(FL_COURIER);
 editdata->name->labelsize(FL_NORMAL_SIZE);
 
@@ -633,7 +638,7 @@ if(view->numb_gc > 1) {
 free(view->sequence[num]);
 view->sequence[num] = newseq;
 view->each_length[num] = l;
-view->modif_but_not_saved = TRUE;
+set_seaview_modified(view, TRUE);
 if(l > view->seq_length) {
 	double x;
 	view->seq_length = l;
@@ -689,7 +694,7 @@ if( *(r - 1) != '\n') *(r++) = '\n';
 *r = 0;
 if(view->comments[num] != NULL) free(view->comments[num]);
 view->comments[num] = temp;
-view->modif_but_not_saved = TRUE;
+set_seaview_modified(view, TRUE);
 }
 
 
