@@ -1,36 +1,28 @@
 #ifdef __APPLE__
+#include "seaview.h"
 #include <Carbon/Carbon.h>
 
 #include "seaview.h"
 #include <string.h>
 #include <stdio.h>
 #include <FL/Fl.H>
+#include <FL/x.H>
 #include <FL/Fl_Sys_Menu_Bar.H>
 #include <FL/filename.H>
-#if TARGET_RT_MAC_MACHO
 #include <sys/stat.h>
 #include <unistd.h>
-#endif
 
 // included functions
-int MG_GetInputFName(char *fname, int maxl, char *title);
-char *mac_fname_to_roman(char *in);
+char *mac_fname_to_roman(const char *in);
 void fl_ringbell(int a);
 extern "C" {
-void *memccpy(void *s1, const void *s2, int c, size_t n);
-#if ! TARGET_RT_MAC_MACHO
-void loop_till_program_ends(void *psn);
-void *my_run_program(char *prog, char *arguments, int flavor);
-char *OSXpathtoOS9path(const char *newpath);
-char *get_prog_dir(void);
-#else
 int my_system_macho(char *command, char *base_fname);
+void myKillExternalJob(ProcessSerialNumber psn);
 char *MG_GetBundleResourcesDir(void);
-#endif
 void PtoC(const void *in, void *out);
 void CtoP(const void *in, void *out);
 void MGinit_apple_events(void);
-void add_apropos(char *progname, void *data);
+void add_apropos(const char *progname, void *data);
 void show_apropos(Fl_Widget *, void *);
 Boolean AppleEventsInstalled ();
 OSErr  MyHandleODoc (const AppleEvent *theAppleEvent, AppleEvent* reply, long handlerRefCon);
@@ -39,19 +31,25 @@ OSErr  MyHandleOApp (const AppleEvent *theAppleEvent, AppleEvent *reply, long ha
 OSErr MyGotRequiredParams (const AppleEvent *theAppleEvent);
 void MG_apple_inits(void);
 }
+void set_seaview_modified(SEA_VIEW *view, int ismodified);
 void cre_sys_menu(void );
 void windowmenuitem_callback(Fl_Widget *o, void *data);
 int find_windowmenuitem(Fl_Window *w);
 int add_windowmenuitem(char *name, Fl_Window *w);
 void rename_windowmenuitem(const char *name, int rank);
 void delete_windowmenuitem(int rank);
-
-
+static pascal void MyNavEventProc_Plus (NavEventCallbackMessage callBackSelector,
+						NavCBRecPtr callBackParms, NavCallBackUserData callBackUD);
+char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int);
 extern void use_initial_file(SEA_VIEW *view, char *masename);
 extern char *get_next_help_line(void *in, char *line, int lline);
+extern int printout_block, printout_fontsize;
+extern int printout_vary, printout_black;
+extern paperformat printout_pageformat;
 
 #elif defined(WIN32)
 
+#include <FL/Fl.H>
 #include <stdarg.h>
 #include <Windows.h>
 #include <string.h>
@@ -62,88 +60,34 @@ extern char *get_next_help_line(void *in, char *line, int lline);
 /* included functions */
 extern "C" {
 void my_bell(void);
-void *memccpy(void *s1, const void *s2, int c, size_t n);
 int mysystem(const char *command);
 char *get_prog_dir(void);
-void add_to_path(char *dir);
 }
-char *MG_win32_file_chooser(const char* message, // title of dialog window
-        const char* pattern, // optional "mase format\0*.mase\0MSF format\0*.msf\0"
-        const char* fname, // default file name
-        const char* defext, // default filename extension or NULL
-        bool save, // TRUE iff output file selection
-        int *pfilterindex); // (inout) pointer to index (from 1) in series of file types in pattern
+char **getutf8args(int *pargc);
+extern char *wchartoutf8(const wchar_t *in);
 #endif
 
 
-
-void *memccpy(void *s1, const void *s2, int c, size_t n)
-{
-char *p, *q;
-p = (char *)s1; q = (char *)s2;
-while(n-- > 0) {
-	*p = *(q++);
-	if( *(p++) == c) return p;
-	}
-return 0;
-}
-
-
 #ifdef __APPLE__
 
+void utf8_Fl_Window::label(const char *l)
+{
+	Fl_Widget::label(l);
+	if(shown()) {
+		CFStringRef mlabel = CFStringCreateWithCString(NULL, l, kCFStringEncodingUTF8);
+		SetWindowTitleWithCFString(fl_xid(this), mlabel);
+		CFRelease(mlabel);
+		}
+}
+
+
 void fl_ringbell(int a)
 {
-SysBeep(a);
+AlertSoundPlay();
 }
 
 
-int MG_GetInputFName(char *fname, int maxl, char *title)
-{
-  OSStatus          err;
-  NavDialogCreationOptions  options;
-  NavDialogRef        navDialog;
-  NavReplyRecord        navReply;
-int         rsult = FALSE;
-FSRef fileRef;
-
-  err = NavGetDefaultDialogCreationOptions( &options );
-  if(err != noErr) return FALSE;
-  options.optionFlags &= !kNavAllowMultipleFiles;
-  options.windowTitle = CFStringCreateWithCString(NULL, title, kCFStringEncodingMacRoman);
-  err = NavCreateChooseFileDialog( &options, NULL, NULL, NULL, NULL, NULL, &navDialog );
-  if(err != noErr) return FALSE;
-
-  // choose the file
-  err = NavDialogRun( navDialog );
-  CFRelease(options.windowTitle);
-  if(err != noErr) return FALSE;
-  
-  err = NavDialogGetReply( navDialog, &navReply );
-  
-  // create url to chosen file
-  if (err == noErr && navReply.validRecord )
-  {    
-	err = AEGetNthPtr( &navReply.selection, 1, typeFSRef, NULL, NULL, &fileRef, sizeof( FSRef ), NULL );
-    if(err == noErr) {
-				err = FSRefMakePath(&fileRef, (UInt8 *)fname, maxl);
-				if (err == noErr) rsult = TRUE;
-                }
-  }  
-  NavDisposeReply( &navReply );
-  NavDialogDispose( navDialog );
-  return rsult;
-}
-
-
-char* MAC_file_chooser(const char* message, const char* pat, const char* fname)
-{
-	static char pathname[FL_PATH_MAX];
-	if( MG_GetInputFName(pathname, FL_PATH_MAX, (char *)message) ) return pathname;
-	else return NULL;
-}
-
-
-char *mac_fname_to_roman(char *in)
+char *mac_fname_to_roman(const char *in)
 /* passage codage pathname vers codage MacRoman qui semble ncessaire pour display cran 
 */
 {
@@ -155,13 +99,10 @@ mycfs = CFStringCreateWithCString(NULL, in, kCFStringEncodingUTF8);
 /* et que MacRoman est utilis pour display */
 ok = CFStringGetCString(mycfs, out, sizeof(out), kCFStringEncodingMacRoman);
 CFRelease(mycfs);
-return (ok ? out : in);
+return (ok ? out : (char *)in);
 }
 
 
-
-#if TARGET_RT_MAC_MACHO
-
 int my_system_macho(char *command, char *base_fname)
 {
 char comfile[300];
@@ -214,13 +155,66 @@ if(status == noErr) {
 	status = fl_choice(
 		"Wait for alignment completion in Terminal window.",
 		"Ignore or interrupt alignment", "Alignment completed", NULL) ? 0 : 1 ; 
-	if( cankill) KillProcess(&newpsn);
+	if(cankill) myKillExternalJob(newpsn);
 	}
 unlink(comfile);
 return status != noErr;
 }
 
 
+pid_t get_desc_process(pid_t ascend)
+//returns the pid of the first descending process from process of pid ascend
+//or 0 if no descendant
+{
+FILE *pipe;
+char command[] = "/bin/ps -U $USER -o pid,ppid ";
+char line[2000], *p;
+long long pid, ppid;
+
+pipe = popen(command, "r");
+fgets(line, sizeof(line), pipe); // skip first line
+while(TRUE)	{
+	p = fgets(line, sizeof(line), pipe);
+	pid = 0;
+	if(p == NULL) break;
+	sscanf(line, "%lld%lld", &pid, &ppid);
+	if( (pid_t)ppid == ascend) break;
+	}
+pclose(pipe);
+return (pid_t) pid;
+}
+
+
+void kill_last_desc(pid_t ascend)
+//kills the deepest descendant process from process ascend, if any
+{
+pid_t son, father;
+
+father = ascend;
+do {
+	son = get_desc_process(father);
+	if(son != 0) father = son;
+	}
+while(son != 0);
+if(father != ascend) kill(father, SIGKILL);
+}
+
+
+void myKillExternalJob(ProcessSerialNumber psn)
+//kills job psn (and its descendant on 10.5)
+{
+pid_t pid;
+long version;
+
+Gestalt(gestaltSystemVersion, &version);
+if(version >= 0x1050) {//on 10.5 must kill separately the job process
+	GetProcessPID(&psn, &pid);
+	kill_last_desc(pid);
+	}
+KillProcess(&psn);
+}
+
+
 char *MG_GetBundleResourcesDir(void)
 {
 ProcessSerialNumber psn;
@@ -234,131 +228,191 @@ strcat(fname, "/Contents/Resources");
 return fname;
 }
 
-#else
-
-void loop_till_program_ends(void *psn)
+static pascal void MyNavEventProc_Plus (NavEventCallbackMessage callBackSelector,
+										NavCBRecPtr callBackParms,
+										NavCallBackUserData callBackUD)
 {
-ProcessInfoRec pinfo;
-OSErr err;
-
-while(1) {
-    Fl::wait();
-    pinfo.processInfoLength = sizeof(ProcessInfoRec);
-    pinfo.processName = NULL;
-    pinfo.processAppSpec = NULL;
-    err = GetProcessInformation((ProcessSerialNumber *)psn, &pinfo);
-    if(err == procNotFound) break;
-  }
-}
-
-
-void *my_run_program(char *prog, char *arguments, int flavor)
-{
-LaunchParamBlockRec pbloc;
-FSSpec fspec;
-OSErr err;
-char pfname[256];
-ProcessSerialNumber *pinfo;
-ScrapRef scrapref;
-OSStatus etat;
-Size l;
-UInt32 count;
-
-if(arguments != NULL) { // put arguments in the scrap under the specified flavor
-	etat = ClearCurrentScrap();  // indispensable !
-	etat = GetCurrentScrap(&scrapref);
-	l = strlen(arguments);
-	etat = PutScrapFlavor(scrapref, flavor, kScrapFlavorMaskNone, l, arguments);
+	OSErr err;
+	static Handle gDitlList = NULL;
+	static DialogItemIndex index;
+	static DialogRef ref;
+	static ControlRef var_control = NULL;
+	Rect rect; Handle h; DialogItemType dtype; Str255 text; ControlRef control;
+	char text2[256];
+	
+	if(callBackSelector == kNavCBCustomize) {
+		if(callBackParms->customRect.bottom == 0) 
+			callBackParms->customRect.bottom = callBackParms->customRect.top + 58;
+		if(callBackParms->customRect.right == 0) 
+			callBackParms->customRect.right = callBackParms->customRect.left + 425;
 	}
-strcpy(pfname + 1, prog );
-pfname[0] = strlen(pfname+1);
-err = FSMakeFSSpec(0, 0, (ConstStr255Param)pfname, &fspec);
-if(err != noErr) return NULL;
-
-pbloc.launchControlFlags = launchContinue | launchNoFileFlags ;
-pbloc.launchBlockID = extendedBlock;
-pbloc.launchEPBLength = extendedBlockLen;
-pbloc.launchAppParameters = NULL;
-pbloc.launchAppSpec = &fspec;
-err = LaunchApplication(&pbloc);
-pinfo = (ProcessSerialNumber *)malloc(sizeof(ProcessSerialNumber));
-*pinfo = pbloc.launchProcessSN;
-return ( err == noErr ? pinfo : NULL) ;
-}
-
-
-static void MG_FSSpecToPathname (FSSpec *myFSS, char *fname, int maxl)
-{
-FSRef myFSRef, pFSRef;
-FSCatalogInfo myinfo;
-static int  anErr = noErr;
-char *p, *q;
-static char buffer[1000], name[256];
-FSSpec fsspec;
-
-p = buffer;
-anErr = FSpMakeFSRef(myFSS, &myFSRef);
-while(1) {
-	anErr = FSGetCatalogInfo(&myFSRef, kFSCatInfoParentDirID, &myinfo, NULL, &fsspec, &pFSRef);
-	PtoC(fsspec.name, name);
-	/* add the file or dir name to the end of buffer in reverse orientation */
-	q = name + strlen(name) - 1;
-	while(q >= name) { *p = *q; q--; p++; }
-	if(myinfo.parentDirID == fsRtParID) break; /* detect when top level is reached */
-	*p = ':'; p++;
-	myFSRef = pFSRef;
+	else if(callBackSelector == kNavCBStart) {
+		gDitlList = GetResource ('DITL', 128);
+		if(gDitlList != NULL) err = NavCustomControl(callBackParms->context, 
+													 kNavCtlAddControlList, gDitlList);
+		err = NavCustomControl(callBackParms->context, kNavCtlGetFirstControlID, &index);
+		index++;
+		ref = GetDialogFromWindow(callBackParms->window);
+		sprintf(text2,"%2d", printout_block);
+		CtoP(text2, text);
+		GetDialogItem(ref, index + 1, &dtype, &h, &rect);
+		SetDialogItemText(h, text);
+		sprintf(text2,"%2d", printout_fontsize);
+		CtoP(text2, text);
+		GetDialogItem(ref, index + 3, &dtype, &h, &rect);
+		SetDialogItemText(h, text);
+		if(printout_black == PDF_COLOR) {
+			err = GetDialogItemAsControl(ref, index + 4, &control);
+			if(err == 0) SetControl32BitValue(control, 1);
+		}
+		else if(printout_black == PDF_BW) {
+			err = GetDialogItemAsControl(ref, index + 8, &control);
+			if(err == 0) SetControl32BitValue(control, 1);
+		}
+		else if(printout_black == TEXT_ONLY) {
+			err = GetDialogItemAsControl(ref, index + 9, &control);
+			if(err == 0) SetControl32BitValue(control, 1);
+		}
+		
+		err = GetDialogItemAsControl(ref, index + 5, &control);
+		if(err == 0) SetControl32BitValue(control, printout_pageformat == 0);
+		err = GetDialogItemAsControl(ref, index + 6, &control);
+		if(err == 0) SetControl32BitValue(control, printout_pageformat == 1);
+		
+		err = GetDialogItemAsControl(ref, index + 7, &var_control);
+		if(err == 0) SetControl32BitValue(var_control, printout_vary);
+		if(*(int *)callBackUD) ActivateControl(var_control);
+		else DeactivateControl(var_control);
 	}
-*p = 0;
-/* invert buffer into fname */
-maxl--;
-if(maxl > strlen(buffer)) maxl = strlen(buffer);
-q = buffer + maxl - 1;
-p = fname;
-while(q >= buffer) { *p = *q; q--; p++; }
-fname[maxl] = 0;
+	else if(callBackSelector == kNavCBTerminate) {
+		if(gDitlList != NULL) {
+			GetDialogItem(ref, index + 1, &dtype, &h, &rect);
+			GetDialogItemText(h, text);
+			PtoC(text, text2);
+			sscanf(text2,"%d", &printout_block);
+			GetDialogItem(ref, index + 3, &dtype, &h, &rect);
+			GetDialogItemText(h, text);
+			PtoC(text, text2);
+			sscanf(text2,"%d", &printout_fontsize);
+			ReleaseResource( gDitlList );
+		}
+		var_control = NULL;
+	}
+	else if(callBackSelector == kNavCBEvent ) {
+		if(var_control != NULL && callBackParms->eventData.itemHit == index + 7) {
+			printout_vary = (printout_vary + 1) % 2;
+			SetControl32BitValue(var_control, printout_vary);
+		}
+		else if(callBackParms->eventData.itemHit == index + 4) {
+			if(printout_black != PDF_COLOR) {
+				printout_black = PDF_COLOR;
+				GetDialogItemAsControl(ref, index + 4, &control);
+				SetControl32BitValue(control, 1);
+				GetDialogItemAsControl(ref, index + 8, &control);
+				SetControl32BitValue(control, 0);
+				GetDialogItemAsControl(ref, index + 9, &control);
+				SetControl32BitValue(control, 0);
+			}
+		}
+		else if(callBackParms->eventData.itemHit == index + 8) {
+			if(printout_black != PDF_BW) {
+				printout_black = PDF_BW;
+				GetDialogItemAsControl(ref, index + 4, &control);
+				SetControl32BitValue(control, 0);
+				GetDialogItemAsControl(ref, index + 8, &control);
+				SetControl32BitValue(control, 1);
+				GetDialogItemAsControl(ref, index + 9, &control);
+				SetControl32BitValue(control, 0);
+			}
+		}
+		else if(callBackParms->eventData.itemHit == index + 9) {
+			if(printout_black != TEXT_ONLY) {
+				printout_black = TEXT_ONLY;
+				GetDialogItemAsControl(ref, index + 4, &control);
+				SetControl32BitValue(control, 0);
+				GetDialogItemAsControl(ref, index + 8, &control);
+				SetControl32BitValue(control, 0);
+				GetDialogItemAsControl(ref, index + 9, &control);
+				SetControl32BitValue(control, 1);
+			}
+		}
+		else if(callBackParms->eventData.itemHit == index + 5) {
+			if(printout_pageformat == LETTER) {
+				GetDialogItemAsControl(ref, index + 5, &control);
+				SetControl32BitValue(control, 1);
+				GetDialogItemAsControl(ref, index + 6, &control);
+				SetControl32BitValue(control, 0);
+				printout_pageformat = A4;
+			}
+		}
+		else if(callBackParms->eventData.itemHit == index + 6) {
+			if(printout_pageformat == A4) {
+				GetDialogItemAsControl(ref, index + 6, &control);
+				SetControl32BitValue(control, 1);
+				GetDialogItemAsControl(ref, index + 5, &control);
+				SetControl32BitValue(control, 0);
+				printout_pageformat = LETTER;
+			}
+		}
+	}
+	return;
 }
 
 
-char *get_prog_dir(void)
-/* returns the pathname of the directory containing the running program
-*/
+char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_only_button)
 {
-FSSpec fspec;
-ProcessInfoRec info;
-ProcessSerialNumber psn;
-OSErr err;
-static char myname[300];
-char *p;
-
-info.processInfoLength = sizeof(ProcessInfoRec);
-info.processName = NULL;
-info.processAppSpec = &fspec;
-err = GetCurrentProcess(&psn);
-if(err != noErr) return NULL;
-err = GetProcessInformation(&psn, &info);
-if(err != noErr) return NULL;
-MG_FSSpecToPathname(&fspec, myname, sizeof(myname) );
-p = strrchr(myname, ':');
-if(p == NULL) return NULL;
-*p = 0;
-return myname;
+    OSErr               anErr = noErr;
+    NavReplyRecord      reply;
+    NavDialogRef        navDialog;
+    NavDialogCreationOptions    dialogOptions;
+    OSType              fileTypeToSave = 'TEXT';
+    OSType              creatorType;
+    NavEventUPP         eventProc = NewNavEventUPP (MyNavEventProc_Plus);
+	char *rsult = NULL;
+	static int *p_use;
+    Boolean ok;
+	static char pathname[FL_PATH_MAX];
+	
+    anErr = NavGetDefaultDialogCreationOptions (&dialogOptions);
+	if(anErr != noErr) return FALSE;
+    dialogOptions.windowTitle = CFStringCreateWithCString(NULL, message, kCFStringEncodingMacRoman);
+    dialogOptions.optionFlags |= kNavNoTypePopup;
+    dialogOptions.optionFlags &= ~ kNavAllowStationery;
+    dialogOptions.saveFileName = CFStringCreateWithCString(NULL, dfault, kCFStringEncodingUTF8); 
+	
+	creatorType = kNavGenericSignature;
+	p_use = &use_only_button;
+	anErr = NavCreatePutFileDialog( &dialogOptions, fileTypeToSave, creatorType, eventProc, p_use, &navDialog );
+	CFRelease(dialogOptions.saveFileName);
+	if(anErr != noErr) return NULL;
+	
+	anErr = NavDialogRun( navDialog );
+	if(anErr != noErr) return NULL;
+	
+	anErr = NavDialogGetReply( navDialog, &reply );
+	
+	if (anErr == noErr && reply.validRecord) {
+		FSRef fileRef;
+		CFURLRef directoryURL, fileURL;
+		
+		anErr = AEGetNthPtr( &reply.selection, 1, typeFSRef, NULL, NULL, &fileRef, sizeof( FSRef ), NULL );
+		if (anErr == noErr)
+		{
+			directoryURL = CFURLCreateFromFSRef( kCFAllocatorDefault, &fileRef );
+			// append new filename to this path
+			fileURL = CFURLCreateCopyAppendingPathComponent( kCFAllocatorDefault, directoryURL, reply.saveFileName, false );
+			ok = CFURLGetFileSystemRepresentation(fileURL, true, (UInt8 *)pathname, FL_PATH_MAX);
+			CFRelease(fileURL); CFRelease(directoryURL);
+			if(ok) rsult = pathname;
+		}
+	}
+	DisposeNavEventUPP(eventProc);
+	NavDisposeReply( &reply );
+	NavDialogDispose( navDialog );
+	return rsult;
 }
 
 
-char *OSXpathtoOS9path(const char *OSXpath /* syntax /disk/dir/fname */)
-{
-static char OS9path[1000];
-FSRef myref;
-FSSpec fsspec;
-
-FSPathMakeRef((const unsigned char *)OSXpath, &myref, NULL);
-FSGetCatalogInfo(&myref, 0, NULL, NULL, &fsspec, NULL);
-MG_FSSpecToPathname(&fsspec, OS9path, sizeof(OS9path));
-return OS9path; /* syntax disk:dir:fname */
-}
-#endif
-
-
 void PtoC(const void *in, void *out)
 {
 char *vin = (char *)in;
@@ -402,26 +456,28 @@ void MGinit_apple_events(void)
 }
 
 
-void add_apropos(char *progname, void *data)
+void add_apropos(const char *progname, void *data)
 {  
-static Fl_Menu_Item item = {"", 0, show_apropos, 0, 0};
-OSStatus err;
-MenuRef mr;
-char title[256];
-Str255 ptitle;
+	static Fl_Menu_Item item = {"", 0, show_apropos, 0, 0};
+	OSStatus err;
+	MenuRef mr;
+	const  char apple_glyph[2] = "\024";
 
-const unsigned char apple_glyph[2] = "\p\024";
-
-Fl_Sys_Menu_Bar *smb = new Fl_Sys_Menu_Bar(0,0,0,0);
-Fl_Menu_Item empty = {0, 0, NULL, 0, 0};
-smb->menu( &empty ); // indispensable pour initialiser fl_sys_menu_bar
-mr = NewMenu(1, apple_glyph);
-InsertMenu(mr, 0);
-sprintf(title, "About %s", progname);
-CtoP(title, ptitle);
-err = InsertMenuItemText(mr, ptitle, 0);
-err = SetMenuItemRefCon(mr, 1, (UInt32) (&item));
-item.user_data_ = data;
+	Fl_Sys_Menu_Bar *smb = new Fl_Sys_Menu_Bar(0,0,0,0);
+	Fl_Menu_Item empty = {0, 0, NULL, 0, 0};
+	smb->menu( &empty ); // indispensable pour initialiser fl_sys_menu_bar
+	err = CreateNewMenu(1, 0, &mr);
+	CFStringRef cftitle = CFStringCreateWithCString(NULL, apple_glyph, kCFStringEncodingMacRoman);
+	err = SetMenuTitleWithCFString(mr, cftitle);
+	CFRelease(cftitle);
+	InsertMenu(mr, 0);
+	CFStringRef format = CFStringCreateWithCString(NULL, "About %s", kCFStringEncodingMacRoman);
+	cftitle = CFStringCreateWithFormat(NULL, NULL, format, progname);
+	CFRelease(format);
+	err = InsertMenuItemTextWithCFString(mr, cftitle, 0, 0,0);
+	CFRelease(cftitle);
+	err = SetMenuItemRefCon(mr, 1, (UInt32) (&item));
+	item.user_data_ = data;
 }
 
 
@@ -462,7 +518,7 @@ int add_windowmenuitem(char *name, Fl_Window *w)
 {
 MenuRef ref = GetMenuRef(2);
 MenuItemIndex rank;
-CFStringRef mycfs = CFStringCreateWithCString(NULL, (name == NULL ? "<empty>" : name), kCFStringEncodingMacRoman);
+CFStringRef mycfs = CFStringCreateWithCString(NULL, (name == NULL ? "<empty>" : name), kCFStringEncodingUTF8);
 AppendMenuItemTextWithCFString(ref, mycfs, 0, 0, &rank);
 CFRelease(mycfs);
 Fl_Menu_Item *item = (Fl_Menu_Item *)calloc(1, sizeof(Fl_Menu_Item));
@@ -475,8 +531,9 @@ return rank;
 void rename_windowmenuitem(const char *name, int rank)
 {
 MenuRef ref = GetMenuRef(2);
-CFStringRef mycfs = CFStringCreateWithCString(NULL, name, kCFStringEncodingMacRoman);
+CFStringRef mycfs = CFStringCreateWithCString(NULL, name, kCFStringEncodingUTF8);
 SetMenuItemTextWithCFString(ref, rank, mycfs);
+CFRelease(mycfs);
 }
 
 void delete_windowmenuitem(int rank)
@@ -497,13 +554,9 @@ if(about == NULL) {
 	Fl_Browser *br = new Fl_Browser(1, 1, about->w() - 2, about->h() - 2);
 	about->end();
 	br->add("");
-#if TARGET_RT_MAC_MACHO
 	p = get_full_path((char *)data);
 	data = (void *)fopen(p, "r");
 	if(data == NULL) return;
-#else
-	get_next_help_line(data, NULL, 0);
-#endif
 	get_next_help_line(data, line, sizeof(line));
 	if(strncmp(line, ">>>", 3) != 0) br->add(line);
 	while(TRUE) {
@@ -513,11 +566,7 @@ if(about == NULL) {
 		while( (p=strchr(line,'\n')) != NULL) *p = 0;
 		br->add(line);
 		}
-#if TARGET_RT_MAC_MACHO
 	fclose((FILE*)data);
-#else
-	get_next_help_line(data, NULL, 0);
-#endif
 	}
 about->show();
 }
@@ -630,10 +679,8 @@ void MG_apple_inits(void)
 #ifdef __APPLE_QUARTZ__
 Fl::set_font(FL_COURIER,"Courier");
 Fl::set_font(FL_COURIER_BOLD,"Courier Bold");
-/* no Courier italic , stay with default Courier New
-Fl::set_font(FL_COURIER_ITALIC,"Courier Italic");
-Fl::set_font(FL_COURIER_BOLD_ITALIC,"Courier Bold Italic");
-*/
+Fl::set_font(FL_COURIER_ITALIC,"Courier Oblique");
+Fl::set_font(FL_COURIER_BOLD_ITALIC,"Courier Bold Oblique");
 #elif defined(__APPLE_QD__)
 Fl::set_font(FL_COURIER," Courier");
 Fl::set_font(FL_COURIER_BOLD,"BCourier");
@@ -644,16 +691,90 @@ error
 #endif
 }
 
+
+void set_seaview_modified(SEA_VIEW *view, int ismodified)
+{
+view->modif_but_not_saved = ismodified;
+SetWindowModified(fl_xid(view->dnawin), (Boolean)ismodified);
+}
 #endif
 
 
 
 #ifdef WIN32
 
+int mysystem(const char *command)
+/*
+command contains "cmd.exe /c prog.exe args"  or  "prog.exe args"
+the first form is compulsory if args contain "> outfile"
+prog is searched in dir where calling program was launched, current dir,
+windows system dir, windows dir, dirs in PATH.
+returns 0 iff OK
+*/
+{
+static char buf[100];
+STARTUPINFO info;
+PROCESS_INFORMATION pi;
+int retval;
 
-void add_to_path(char *dir)
-/* ajoute a la variable PATH la directory dir
-teste si deja dedans ou si vide */
+GetStartupInfo(&info);
+retval = CreateProcess(NULL, (char *)command,0,0,0,0,0,_getcwd(buf,sizeof(buf)),&info,&pi);
+if(retval == 0) return 1;
+do	{
+	Fl::check();
+	retval = WaitForSingleObject(pi.hProcess, 500);
+	}
+while(retval == WAIT_TIMEOUT);
+return 0;
+}
+
+char *get_prog_dir(void)
+/* returns path of dir where calling program was launched */
+{
+static char *dir = NULL;
+char *line, *p;
+int l;
+	
+if(dir == NULL) {
+	line = wchartoutf8(GetCommandLineW());
+	//fprintf(OUT,"retour GetCommandLine line=%s\n",line);fflush(OUT);
+	/* extract 1st word of line or first group delimited by " " */
+	if(*line == '"') { line++; p = strchr(line, '"'); }
+	else p = strchr(line, ' '); 
+	if(p == NULL) p = line + strlen(line);
+	l = p - line;
+	while( l > 0 && line[l - 1] != '\\' ) l--;
+	if(l > 0) {
+		dir = (char *)malloc(l + 1);
+		memcpy(dir, line, l); dir[l] = 0;
+		}
+	}
+return dir;
+}
+
+
+void my_bell(void)
+{
+Beep(0,0);
+} 
+
+
+char **getutf8args(int *pargc)
+{
+int argc;
+wchar_t **wargv = CommandLineToArgvW(GetCommandLineW(), &argc);
+char **argv = (char **)malloc(argc * sizeof(char *));
+for (int count = 0; count < argc; count++) {
+	argv[count] = strdup(wchartoutf8(wargv[count]));
+	}
+*pargc = argc;
+return argv;
+}
+
+/*
+ void add_to_path(char *dir)
+ /* ajoute a la variable PATH la directory dir
+ teste si deja dedans ou si vide *
 {
 char *buffer;
 int l;
@@ -662,105 +783,14 @@ if(dir == NULL || strlen(dir) == 0) return;
 buffer = (char *)malloc(10000);
 l = GetEnvironmentVariable("PATH", buffer, 10000);
 if(l == 0) *buffer = 0;
-//fprintf(OUT,"l=%d PATH=%s\n",l,buffer);fflush(OUT);
 if(strstr(buffer, dir) != NULL) return;
 if(*buffer != 0) strcat(buffer, ";");
 strcat(buffer, dir);
 SetEnvironmentVariable("PATH", buffer);
-
-//GetEnvironmentVariable("PATH", buffer, sizeof(buffer));
-//fprintf(OUT,"PATH=%s\n",buffer);fflush(OUT);
-
 free(buffer);
 }
 
-static FILE *out=NULL;
 
-int mysystem(const char *command)
-/*
-command contains "cmd.exe /c prog.exe args"  or  "prog.exe args"
-the first form is compulsory if args contain "> outfile"
-prog is searched in dir where calling program was launched, current dir,
-windows system dir, windows dir, dirs in PATH.
-*/
-{
-static char buf[100];
-STARTUPINFO info;
-PROCESS_INFORMATION pi;
-int retval;
-
-//if(out==NULL)out=fopen("seaview.log","w");
-GetStartupInfo(&info);
-//fprintf(OUT,"retour GetStartupInfo \n");fflush(OUT);
-retval = CreateProcess(NULL, (char *)command,0,0,0,0,0,_getcwd(buf,sizeof(buf)),&info,&pi);
-retval = WaitForSingleObject(pi.hProcess, INFINITE);
-//fprintf(out,"retour WaitForSingleObject retval=%d WAIT_FAILED=%d\n",retval,WAIT_FAILED);fflush(out);
-return (retval == WAIT_FAILED);
-}
-
-char *get_prog_dir(void)
-/* returns path of dir where calling program was launched */
-{
-static char dir[100];
-char *line, *p;
-int l;
-
-line = GetCommandLine();
-//fprintf(OUT,"retour GetCommandLine line=%s\n",line);fflush(OUT);
-/* extract 1st word of line or first group delimited by " " */
-if(*line == '"') { line++; p = strchr(line, '"'); }
-else
-	p = strchr(line, ' '); 
-if(p == NULL) p = line + strlen(line);
-l = p - line;
-while( l > 0 && line[l - 1] != '\\' ) l--;
-if(l <= 0) return NULL;
-memcpy(dir, line, l); dir[l] = 0;
-return dir;
-}
-
-
-void my_bell(void)
-{
-Beep(0,0);
-}
-
-char *MG_win32_file_chooser(const char* message, // title of dialog window
-        const char* pattern, // optional "mase format\0*.mase\0MSF format\0*.msf\0"
-        const char* fname, // default file name
-        const char* defext, // default filename extension or NULL
-        bool save, // TRUE iff output file selection
-        int *pfilterindex) // (inout) pointer to index (from 1) in series of file types in pattern
- {
-     static char filenamebuffer[MAX_PATH];
-     static OPENFILENAME wreq;
-     bool retval;
-     
-     memset(&wreq, 0, sizeof(wreq));
-     wreq.lStructSize = sizeof(OPENFILENAME); 
-     wreq.lpstrFilter = pattern;
-     if(pattern == NULL) wreq.nFilterIndex = 0;
-     else if(pfilterindex != NULL) wreq.nFilterIndex = *pfilterindex;
-     else wreq.nFilterIndex = 1;
-     if(defext != NULL) wreq.lpstrDefExt = defext;
-     wreq.lpstrFile = filenamebuffer;
-     wreq.nMaxFile = MAX_PATH;
-     wreq.lpstrTitle = message ? message : "Select the filename";
-     if(fname) {
-       memset(filenamebuffer, 0, MAX_PATH);
-       if((filenamebuffer[1] == ':') && (_getdrive() + 'A' - 1 == filenamebuffer[0]))
-   strncpy(filenamebuffer, fname + 2, MAX_PATH);
-       else
-   strncpy(filenamebuffer, fname, MAX_PATH);
-     }
-     wreq.Flags = OFN_NOCHANGEDIR | OFN_HIDEREADONLY;
-     retval = (save ? GetSaveFileName(&wreq) : GetOpenFileName(&wreq));
-     if(save && (pfilterindex != NULL) ) *pfilterindex = wreq.nFilterIndex;
-     return retval == 0 ? NULL : wreq.lpstrFile;
- }
- 
-
-/*
 char **split_args(char *all, int *pargc)
 {
 char *p, *q, **argv;
