@@ -33,7 +33,6 @@ int save_nexus_file(const char *fname, int ntaxa, int protein,
 int nextbracket(FILE *in);
 char *next_token(FILE *in);
 FILE *is_nexus_file(char *nom);
-char *majuscules(char *p);
 char *next_block_name(FILE *in);
 int process_block_taxa(FILE *in, char ***taxnames);
 int is_block_end(char *p);
@@ -133,6 +132,11 @@ p = big_token;
 if(c == APOSTROPHE) {
 	while (1)	{
 		c = fgetc(in);
+		if(c == '\r') {
+			punct = fgetc(in);
+			if(punct != '\n') ungetc(punct, in);
+			c = '\n';
+			}
 		if(c == EOF) break;
 		if(c == APOSTROPHE) {
 			punct = fgetc(in);
@@ -188,7 +192,8 @@ char *p;
 
 in = fopen(fname, "r");
 if(in == NULL) return NULL;
-p = majuscules(next_token(in));
+p = next_token(in);
+if(p != NULL) majuscules(p);
 if(p == NULL || strcmp(p, "#NEXUS") != 0 ) { fclose(in); return NULL; }
 else return in;
 }
@@ -200,12 +205,14 @@ char *p;
 static char name[81];
 
 do	{
-	p = majuscules(next_token(in));
+	p = next_token(in);
 	if(p == NULL) return NULL;
+	majuscules(p);
 	}
 while(strcmp(p, "BEGIN") != 0);
-p =  majuscules(next_token(in));
+p = next_token(in);
 if(p == NULL) return NULL;
+majuscules(p);
 strcpy(name, p);
 next_token(in); /* skip the ; */
 return name;
@@ -214,7 +221,7 @@ return name;
 
 int is_block_end(char *p)
 {
-majuscules(p);
+if(p!=NULL)majuscules(p);
 return p == NULL || strcmp(p, "END") == 0 || strcmp(p, "ENDBLOCK") == 0;
 }
 
@@ -242,8 +249,9 @@ int process_taxa_dim(FILE *in)
 char *p;
 int ntax;
 
-p = majuscules(next_token(in));
+p = next_token(in);
 if(p == NULL) return 0;
+majuscules(p);
 if(strcmp(p, "NTAX") == 0) {
 	next_token(in);
 	p = next_token(in);
@@ -261,8 +269,9 @@ char *p;
 int sites = 0;
 
 do	{
-	p = majuscules(next_token(in));
+	p = next_token(in);
 	if(p == NULL) break;
+	majuscules(p);
 	if(strcmp(p, "NCHAR") == 0) {
 		next_token(in); /* skip = */
 		p = next_token(in);
@@ -290,12 +299,14 @@ int gap;
 *interleave = FALSE;
 
 do	{
-	p = majuscules(next_token(in));
+	p = next_token(in);
 	if(p == NULL) break;
+	majuscules(p);
 	if(strcmp(p, "DATATYPE") == 0) {
 		next_token(in); /* skip = */
-		p = majuscules(next_token(in));
+		p = next_token(in);
 		if(p == NULL) break;
+		majuscules(p);
 		*protein = strcmp(p, "PROTEIN") == 0;
 		}
 	else if(strcmp(p, "GAP") == 0) {
@@ -779,8 +790,9 @@ int num, count;
 
 num = -1;
 do	{
-	p = majuscules(next_token(in));
+	p = next_token(in);
 	if(p == NULL) break;
+	majuscules(p);
 	if(strcmp(p, "TAXON") == 0) {
 		next_token(in); /* skip = */
 		p = next_token(in);
@@ -872,8 +884,9 @@ char *p, *text = NULL;
 
 *name = NULL;
 do	{
-	p = majuscules(next_token(in));
+	p = next_token(in);
 	if(p == NULL) break;
+	majuscules(p);
 	if(strcmp(p, "FOOTER") == 0) {
 		next_token(in); /* skip = */
 		p = next_token(in);
@@ -932,9 +945,11 @@ int process_unal_dim(FILE *in)
 char *p;
 int number = 0;
 
-p = majuscules(next_token(in));
+p = next_token(in);
+if(p!=NULL)majuscules(p);
 if(p == NULL || strcmp(p, "NEWTAXA") != 0) { skip_command(in); return 0; }
-p = majuscules(next_token(in));
+p = next_token(in);
+if(p!=NULL)majuscules(p);
 if(p == NULL || strcmp(p, "NTAX") != 0) { skip_command(in); return 0; }
 next_token(in); /* skip = */
 p = next_token(in);
@@ -950,10 +965,12 @@ int process_unal_format(FILE *in)
 char *p;
 int protein;
 
-p = majuscules(next_token(in));
+p = next_token(in);
+if(p!=NULL)majuscules(p);
 if(p == NULL || strcmp(p, "DATATYPE") != 0) { skip_command(in); return 0; }
 next_token(in); /* skip = */
-p = majuscules(next_token(in));
+p = next_token(in);
+if(p!=NULL)majuscules(p);
 if(p != NULL) protein = (strcmp(p, "PROTEIN") == 0);
 if(p != NULL && *p != ';') skip_command(in);
 return protein;
@@ -1014,7 +1031,7 @@ FILE *in;
 int lseqs, i, tot;
 
 if( (in = is_nexus_file(fname) ) == NULL) {
-	*err_message = "not a NEXUS file";
+	*err_message = (char *)"not a NEXUS file";
 	return 0;
 	}
 tot = read_nexus_file(in, pseqs, &lseqs, protein, pseqnames, 
@@ -1022,7 +1039,7 @@ tot = read_nexus_file(in, pseqs, &lseqs, protein, pseqnames,
 	pnum_species_sets, list_species_sets, name_species_sets,
 	charsets, ptot_comment_lines, comment_name, comment_line);
 if(tot == 0) {
-	*err_message = "format error";
+	*err_message = (char *)"format error";
 	return 0;
 	}
 if(*ptot_comment_lines > 0) {
