@@ -26,7 +26,7 @@ static void insert_in_active_region(SEA_VIEW *view, int numsite, int total);
 void insert_region_part(SEA_VIEW *view, int numsite, int total);
 list_regions *parse_regions_from_header(char *header);
 void draw_region_background(SEA_VIEW *view, int f_seq0, int l_seq0);
-char *create_species_set(SEA_VIEW *view, char *set_name);
+const char *create_species_set(SEA_VIEW *view, char *set_name);
 void species_menu_callback(Fl_Widget *ob, void *extra);
 void get_menu_taille_valeur(Fl_Widget *menu, int *taille, int *valeur);
 void init_menu_taille_valeur(Fl_Widget *menu, int taille, int valeur, 
@@ -822,7 +822,7 @@ while( (segment = segment->next) != NULL );
 }
 
 
-char *create_species_set(SEA_VIEW *view, char *set_name)
+const char *create_species_set(SEA_VIEW *view, char *set_name)
 {
 int newnumb, i;
 if(view->numb_species_sets >= MAX_SPECIES_SETS)
@@ -851,7 +851,8 @@ return NULL;
 void species_menu_callback(Fl_Widget *ob, void *extra)
 {
 SEA_VIEW *view;
-char *set_name, *message;
+char *set_name;
+const char *message;
 int i, rang, numset, taille, reponse = ((Fl_Menu_ *)ob)->value() + 1;
 
 if(reponse <= 0 ) return;
