@@ -31,7 +31,15 @@
 extern "C" { char *my_fgets(char *s, int n, FILE *f); }
 
 #ifdef WIN32
-extern "C" { void my_bell(void); }
+#include <wchar.h>
+typedef const wchar_t *LPCWSTR;
+extern char *wchartoutf8(LPCWSTR in);
+extern LPCWSTR utf8towchar(const char *in);
+extern "C" { 
+	void my_bell(void); 
+}
+#define fopen(a,b) _wfopen(utf8towchar(a), L##b)
+#define memccpy _memccpy
 #define fl_ringbell(a) my_bell()
 #elif defined __APPLE__
 extern void fl_ringbell(int);
@@ -43,11 +51,19 @@ extern void fl_ringbell(int);
 #define FL_min(a,b)      ( (a) < (b) ? (a):(b) )
 #ifdef __APPLE__
 #define PREPARE_LABEL(p) mac_fname_to_roman(p)
+extern void set_seaview_modified(struct _SEA_VIEW *view, int ismodified);
+// subclass Fl_Window and redefine its label(const char *) member function to accept any UTF8 string
+class FL_EXPORT utf8_Fl_Window :  public Fl_Window {
+public:
+	utf8_Fl_Window(int W, int H) : Fl_Window(W,H) {}
+	void label(const char *label);
+	const char* label() const {return Fl_Widget::label();}
+};
 #else
 #define PREPARE_LABEL(p) p
+#define set_seaview_modified(view, ismodif) view->modif_but_not_saved = ismodif
+#define utf8_Fl_Window Fl_Window
 #endif
-/* proto manque parfois */
-extern "C" { void *memccpy(void *s1, const void *s2, int c, size_t n); }
 typedef struct _user_data_plus {
 	void *p;
 	int value;
@@ -57,7 +73,6 @@ extern void fl_reset_cursor(Fl_Window *win);
 
 #define MAXLENCOM 50000 /* long max des commentaires sous mase */
 #define MAX_SPECIES_SETS 50 /* nbre max de species sets */
-#define CLU_WID_NAME 16
 #define MSF_WID_NAME 15
 #define CLU_BLOCK_LEN 5000 /* block pour allocation mem format Clustal */
 #define MAX_GAP_SITES 1000
@@ -89,7 +104,7 @@ typedef enum {MASE_FORMAT, PHYLIP_FORMAT, CLUSTAL_FORMAT, MSF_FORMAT,
 	known_format;
 
 typedef enum {OPEN_ANY, OPEN_MASE, OPEN_PHYLIP, OPEN_CLUSTAL, OPEN_MSF, OPEN_FASTA,
-	OPEN_NEXUS, SAVE, SAVE_AS, SAVE_REGIONS, SAVE_PROT_ALIGN, PRINTOUT, 
+	OPEN_NEXUS, ACNUC_IMPORT, SAVE, SAVE_AS, SAVE_REGIONS, SAVE_PROT_ALIGN, PRINTOUT, 
 #ifndef __APPLE__
 	PDFOPTIONS,
 #endif
@@ -99,8 +114,8 @@ typedef enum {OPEN_ANY, OPEN_MASE, OPEN_PHYLIP, OPEN_CLUSTAL, OPEN_MSF, OPEN_FAS
 typedef enum {NO_ALT_COLORS, USING_STANDARD_COLORS, USING_ALT_COLORS}
 	color_choice;
 
-typedef enum {COPY_SEQS, PASTE_SEQS, RENAME_SEQ, EDIT_COMMENTS, EDIT_SEQ, DELETE_SEQ, CREATE_SEQ, LOAD_SEQ, 
-	DUPLICATE_SEQ,
+typedef enum {COPY_SEQS, PASTE_SEQS, RENAME_SEQ, EDIT_COMMENTS, EDIT_SEQ, DELETE_SEQ, CREATE_SEQ, 
+	LOAD_SEQ, DUPLICATE_SEQ,
 	COMPLEMENT_SEQ, REVERSE_SEQ, EXCHANGE_UT, ALIGN_ALL_SEQS, ALIGN_SELECTION, DOT_PLOT,
 	CONSENSUS_SEQ, DELETE_GAP_ONLY_SITES, SET_GCODE}
 	edit_choice;
@@ -112,7 +127,7 @@ typedef enum {A4, LETTER} paperformat;
 
 typedef enum {PDF_COLOR, PDF_BW, TEXT_ONLY} pdf_kind;
 
-typedef struct {
+typedef struct _SEA_VIEW {
 	Fl_Widget *DNA_obj; /* free object for names + seqs */
 	Fl_Scrollbar *horsli; /* horizontal scrollbar */
 	Fl_Scrollbar *vertsli; /* vertical scrollbar */
@@ -180,8 +195,8 @@ typedef struct {
 	int allow_seq_edit; /* when TRUE can type sequence data in */
 	void *viewasprots; /* !=NULL when DNA displayed as proteins */
 	int hjkl; /* when TRUE, keys hjklHJKL produce TCGA */
-	int modif_but_not_saved;
-	Fl_Window *dnawin; /* window of DNA_obj */
+	int modif_but_not_saved; /* set/unset through function set_seaview_modified */
+	utf8_Fl_Window *dnawin; /* full seaview window */
 	int double_buffer; /* true iff DNA_obj is double buffered */
 	int alignment_algorithm; /* rank (from 0) of chosen alignment algorithm */
 	int count_msa_algos; /* number of usable alignment algorithms */
@@ -207,17 +222,17 @@ typedef struct {
 	
 
 typedef struct _props_menu_parts {
-	int fontsize, saveformat, colors, viewasprots, inverted, edit, reference, keys,
+	int fontsize, colors, viewasprots, inverted, edit, reference, keys,
 		clustalopt, consensusopt, slow_fast, allow_lower, custom;
 	SEA_VIEW *view;
 	} props_menu_parts;
 
 
 extern void out_of_memory(void);
-extern char *majuscules(char *s);
+extern "C" void majuscules(char *s);
 extern void init_dna_scroller(SEA_VIEW *view, int totseqs, 
 	const char *masename, int protein, char *header);
-extern char *extract_filename( char *fname);
+extern const char *extract_filename(const char *fname);
 extern void draw_region_line(Fl_Widget *ob, SEA_VIEW *view);
 extern void props_button_callback(Fl_Widget *ob, long mainpop);
 extern void regions_menu_callback(Fl_Widget *ob, void *extra);
@@ -259,7 +274,7 @@ extern char *save_alignment_or_region(const char *fname, char **seq,
 	char **name_species_sets, int *sel_seqs, int tot_sel_seqs, int protein,
 	int tot_comment_lines, char **comment_name, char **comment_line, int phylipwidnames);
 extern void load_alignment_file(SEA_VIEW *view, char *filename, 
-	char *message, known_format file_format);
+	const char *message, known_format file_format);
 extern int is_a_protein_seq(char *seq);
 extern int prepare_printout(const char *filename, char **seq, int totseqs,  
 	char **seqname,
@@ -295,7 +310,7 @@ extern int insert_gap_all_comments(int numgaps, int pos,  SEA_VIEW *view);
 extern int delete_in_all_comments(int numdels, int pos,  SEA_VIEW *view);
 extern char* fl_file_chooser(const char* message, const char* pat, const char* fname, int);
 #ifdef __APPLE__
-extern char *mac_fname_to_roman(char *in); 
+extern char *mac_fname_to_roman(const char *in); 
 #endif
 
 #endif
