@@ -1,4 +1,5 @@
 #include "seaview.h"
+#include "FL/Fl_Native_File_Chooser.H"
 #include <ctype.h>
 #include <time.h>
 
@@ -57,20 +58,27 @@ static void save_species_sets(int numb_species_sets, int **list_species_sets,
 void save_comment_lines(int tot_comment_lines, char **names, char **lines, 
 	FILE *out);
 known_format what_format(const char *filename);
-extern "C" {
-	char *my_fgets(char *s, int n, FILE *f);
-	}
+char* seaview_file_chooser_save_as(const char* message, const char* fname, SEA_VIEW *view);
 
 
 
 /* external */
-int save_nexus_file(const char *fname, int ntaxa, int protein,
+extern char *f_format_names[];
+extern char *f_format_exts[];
+extern int nbr_formats;
+
+
+extern int save_nexus_file(const char *fname, int ntaxa, int protein,
 	char **seqs, char **taxnames, char **notes, char *header,
 	int num_species_sets, int **list_species_sets, 
 	char **name_species_sets,
 	list_regions *charsets,
 	int tot_comment_lines, char **comment_name, char **comment_line,
 	region *region_used, int *sel_seqs, int tot_sel_seqs, int *eachlength);
+extern void set_save_format(SEA_VIEW *view, int val);
+extern "C" {
+	char *my_fgets(char *s, int n, FILE *f);
+}
 
 
 
@@ -348,7 +356,8 @@ int read_phylip_align(const char *fname, char ***pseq, char ***pseqname,
 		char ***pcomments, char **pheader, char **err_message)
 {
 FILE *in;
-char *p, *q, line[300];
+char *p, *q;
+static char line[30000];
 char **seq, **comments, **seqname;
 int totseqs, lenseqs, i, l;
 static char ret_message[200];
@@ -935,7 +944,7 @@ for(num=0; num<totseqs; num++) {
 	current = 0; psegment = region_used->list;
 	all_sequence.fin = eachlength[num];
 	fprintf(out,">%s", seqname[num]);
-	if(comments[num] != NULL) {
+	if(comments != NULL && comments[num] != NULL) {
 		putc(' ', out); 
 		p = comments[num] + 1;
 		while(*p != '\n' && *p != 0) putc(*(p++), out);
@@ -1414,16 +1423,34 @@ for(l = nl; l > 0; l--) {
 
 
 extern "C" { 
-#ifdef WIN32
 	char *get_prog_dir(void); 
-#elif defined(__APPLE__)
+#if defined(__APPLE__)
 	char *MG_GetBundleResourcesDir(void);
 #endif
 	}
 
+#if !(defined(WIN32) || defined(__APPLE__))
+static char seaview_prog_dir[200] = "";
+void inform_prog_dir(const char *arg0)
+{
+char *p;
+if((p = strrchr(arg0, '/')) != NULL) {
+	memcpy(seaview_prog_dir, arg0, p - arg0 + 1);
+	seaview_prog_dir[p - arg0 + 1] = 0;
+	}
+else seaview_prog_dir[0] = 0;
+}
+
+char *get_prog_dir(void)
+{
+return seaview_prog_dir;
+}
+#endif
+
 
 char *get_full_path(const char *fname)  
-/* to get full pathname to file fname searching for its name and then for it through all path directories 
+/* to get full pathname to file fname searching for its name, for it in the prog dir
+and then for it through all path directories 
 returns NULL if not found 
 */
 {
@@ -1441,10 +1468,15 @@ FILE *fich;
 int lf, ltot;
 
 strcpy(dir, fname);
-fich = fopen(dir, "r"); /* try first explicit filename */
+if(strchr(fname, DIR_SEPAR) != NULL) {// if fname is a pathname
+	fich = fopen(dir, "r"); /* try first explicit filename */
+	goto way_out; // and don't search more
+	}
+#if defined(__APPLE__)
+sprintf(dir, "%s/%s", MG_GetBundleResourcesDir(), fname);
+fich = fopen(dir, "r");
 if(fich != NULL) goto way_out;
-if(strchr(fname, DIR_SEPAR) != NULL) return NULL;// if fname is a pathname, don't search more
-#ifdef WIN32
+#else
 /* try dir where program was launched */
 deb = get_prog_dir();
 if(deb != NULL) {
@@ -1453,10 +1485,6 @@ if(deb != NULL) {
 	fich = fopen(dir, "r");
 	if(fich != NULL) goto way_out;
 	}
-#elif defined(__APPLE__)
-sprintf(dir, "%s/%s", MG_GetBundleResourcesDir(), fname);
-fich = fopen(dir, "r");
-if(fich != NULL) goto way_out;
 #endif
 path = getenv("PATH"); // get the list of path directories, separated by : or ;
 if (path == NULL ) return NULL;
@@ -1589,3 +1617,25 @@ while(--n > 0) {
 *p = 0;
 return (p == s ? NULL : s);
 }
+
+
+char* seaview_file_chooser_save_as(const char* message, const char* fname, SEA_VIEW *view)
+{
+char *p, types_list[500] = "";
+Fl_Native_File_Chooser *chooser = new Fl_Native_File_Chooser();
+chooser->type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);   
+chooser->title(message);                               
+chooser->preset_file(extract_filename(fname));                      
+	
+p = types_list;
+for(int f = 0; f < nbr_formats; f++) {
+	sprintf(p, "%s\t*.%s\n", f_format_names[f], f_format_exts[f]);
+	p += strlen(p);
+	}
+chooser->filter(types_list);    
+chooser->filter_value(view->format_for_save);    
+if ( chooser->show() != 0) return NULL;
+set_save_format((SEA_VIEW *)view, chooser->filter_value());
+return (char *)chooser->filename();
+}
+
