@@ -20,7 +20,7 @@ gap_site gap_sites[MAX_GAP_SITES];
 void show_dot_plot(char *seq1, char *seq2, char *seqname1, char *seqname2,
 	int l1, int l2, int maxseqlength, void *seaview_data);
 FD_matpt *create_form_matpt(void);
-static void err_message(char *text);
+static void err_message(const char *text);
 static char *check_alloc(int nbrelt, int sizelt);
 void log_to_phys(int lx, int ly, int *px, int *py, FD_matpt *matpt);
 void phys_to_log(int px, int py, int *lx, int *ly, FD_matpt *matpt);
@@ -134,7 +134,7 @@ compute_title(matpt);
 
 
 static Fl_Widget *cre_button(int x, int y, int *w, int h, 
-	int si, char *label)
+	int si, const char *label)
 {
 Fl_Widget *obj;
 fl_font(FL_HELVETICA, si);
@@ -342,8 +342,8 @@ obj->labelfont(FL_COURIER);
 fdui->region_box = obj;
 
 fl_font(fdui->region_box->labelfont(), fdui->region_box->labelsize() );
-int debut = 4 + 10 * (int)fl_width('A');
-int maxbound = (fdui->region_box->w() - 8) / (int)fl_width('A') - 6;
+int debut = 4 + 10 * (int)fl_width("A");
+int maxbound = (fdui->region_box->w() - 8) / (int)fl_width("A") - 6;
 obj = new Fl_Slider(FL_HOR_FILL_SLIDER, fdui->region_box->x() + debut,
 	fdui->region_box->y() + fdui->region_box->h(),
 	fdui->region_box->w() - debut, SQW, "");
@@ -396,7 +396,7 @@ return fdui;
 }
 
 
-static void err_message(char *text)
+static void err_message(const char *text)
 {
 fl_alert(text);
 exit(1);
@@ -483,8 +483,12 @@ matpt->modif_but_not_saved = FALSE;
 matpt->plotname = (char *)malloc(strlen(name) + 1);
 strcpy(matpt->plotname, name);
 if(matpt->align_button != NULL) {
-	char temp[200];
-	sprintf(temp, "align to %s|align to %s", seqname1, seqname2);
+	char temp[200], name1[40], name2[40], *p;
+	sprintf(name1, "%.30s", seqname1);
+	if((p = strchr(name1,'|')) != NULL) *p = 0;
+	sprintf(name2, "%.30s", seqname2);
+	if((p = strchr(name2,'|')) != NULL) *p = 0;
+	sprintf(temp, "align to %s|align to %s", name1, name2);
 	((Fl_Choice*)matpt->choice_ref_seq)->clear();
 	((Fl_Choice *)matpt->choice_ref_seq)->value(0);
 	((Fl_Choice *)matpt->choice_ref_seq)->add(temp);
@@ -831,7 +835,7 @@ reset_mat_data(matpt);
 
 fl_message("Dot plot is now in file\n%s\nin postscript format", plotfname);
 sprintf(plotfname,
-	"Horizontal: %s (%d)  Vertical: %s (%d)  Window: %d Matches: %d",
+	"Horizontal: %.30s (%d)  Vertical: %.30s (%d)  Window: %d Matches: %d",
 	matpt->seqname1, matpt->seqlong1, matpt->seqname2, matpt->seqlong2, 
 		matpt->fenetre, matpt->identites);
 fprintf(plotfile,"-10 -30 moveto (%s) show\n",plotfname);
@@ -916,8 +920,8 @@ matpt->mat_panel->redraw();
 
 void compute_title(FD_matpt *matpt)
 {
-static char title[100];
-sprintf(title, "Horizontal: %s (%d)   Vertical: %s (%d)",
+static char title[150];
+sprintf(title, "Horizontal: %.30s (%d)   Vertical: %.30s (%d)",
 	matpt->seqname1, matpt->seqlong1, matpt->seqname2, matpt->seqlong2);
 matpt->title = title;
 matpt->mat_panel->window()->label(title);
@@ -1131,7 +1135,7 @@ if( need_big_gaps ) {
 	update_current_seq_length(newlength, view);
 	view->horsli->redraw();
 	}
-view->modif_but_not_saved = TRUE;
+set_seaview_modified(view, TRUE);
 matpt->modif_but_not_saved = FALSE;
 view->DNA_obj->redraw();
 fl_reset_cursor(ob->window());
